<?php
set_include_path(get_include_path() . ';' . dirname(dirname(__FILE__)) );
error_reporting(E_ALL^E_NOTICE);

require_once 'simpletest/unit_tester.php';
require_once 'simpletest/reporter.php';

require_once 'daomancy/Daomancy.php';
require_once 'dao/PageDAO.php';
require_once 'dao/CategoryDAO.php';

class DaomancyTestCase extends UnitTestCase
{
	private $dbh;
	
	function setUp()
	{
		$this->dbh = new PDO('sqlite:' . dirname(__FILE__) . '/mydb.db');
	}
	
	function testFrom()
	{
		$this->assertPattern(
			'/^\s*SELECT'
				.'\s+p\.page_id'
				.'\s*,\s*p\.name'
				.'\s*,\s*p\.category_id'
				.'\s*,\s*p\.contents'
				.'\s*,\s*p\.is_active'
				.'\s*,\s*p\.create_date'
				.'\s*,\s*p\.update_date'
				.'\s+FROM\s+page\s+p/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->toSQL()
		);
	}
	
	function testJoin()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'\s+p\.page_id'
			.'\s*,\s*p\.name'
			.'\s*,\s*p\.category_id'
			.'\s*,\s*p\.contents'
			.'\s*,\s*p\.is_active'
			.'\s*,\s*p\.create_date'
			.'\s*,\s*p\.update_date'
			.'\s*,\s*c\.category_id'
			.'\s*,\s*c\.name'
			.'\s*,\s*c\.is_active'
			.'\s*,\s*c\.category_kbn'
			.'\s*,\s*c\.create_date'
			.'\s*,\s*c\.update_date'
			.'\s+FROM\s+page\s+p'
			.'\s+INNER\s+JOIN\s+category\s+c'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->toSQL()
		);
	}
	
	function testLeftJoin()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'\s+p\.page_id'
			.'\s*,\s*p\.name'
			.'\s*,\s*p\.category_id'
			.'\s*,\s*p\.contents'
			.'\s*,\s*p\.is_active'
			.'\s*,\s*p\.create_date'
			.'\s*,\s*p\.update_date'
			.'\s*,\s*c\.category_id'
			.'\s*,\s*c\.name'
			.'\s*,\s*c\.is_active'
			.'\s*,\s*c\.category_kbn'
			.'\s*,\s*c\.create_date'
			.'\s*,\s*c\.update_date'
			.'\s+FROM\s+page\s+p'
			.'\s+LEFT\s+OUTER\s+JOIN\s+category\s+c'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->leftJoin('CategoryDAO' , 'c')
				->toSQL()
		);
	}
	
	function testRightJoin()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'\s+p\.page_id'
			.'\s*,\s*p\.name'
			.'\s*,\s*p\.category_id'
			.'\s*,\s*p\.contents'
			.'\s*,\s*p\.is_active'
			.'\s*,\s*p\.create_date'
			.'\s*,\s*p\.update_date'
			.'\s*,\s*c\.category_id'
			.'\s*,\s*c\.name'
			.'\s*,\s*c\.is_active'
			.'\s*,\s*c\.category_kbn'
			.'\s*,\s*c\.create_date'
			.'\s*,\s*c\.update_date'
			.'\s+FROM\s+page\s+p'
			.'\s+RIGHT\s+OUTER\s+JOIN\s+category\s+c'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->rightJoin('CategoryDAO' , 'c')
				->toSQL()
		);
	}
	
	function testOn()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+FROM\s+page\s+p'
			.'\s+INNER\s+JOIN\s+category\s+c'
			.'\s+ON\s+p.category_id=c.category_id'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->toSQL()
		);
	}
	
	function testOnAnd()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+FROM\s+page\s+p'
			.'\s+INNER\s+JOIN\s+category\s+c'
			.'\s+ON\s+p.category_id=c.category_id'
			.'\s+AND\s+p.category_id=c.category_id'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->toSQL()
		);
	}
	
	function testWhere()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+FROM\s+page\s+p'
			.'\s+INNER\s+JOIN\s+category\s+c'
			.'\s+ON\s+p.category_id=c.category_id'
			.'\s+AND\s+p.category_id=c.category_id'
			.'\s+WHERE\s+p\.name\s+LIKE\s+\?'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->toSQL()
		);
	}
	
	function testWhereParams()
	{
		$this->assertEqual(
			array('ページ名')
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->getParams()
		);
	}
	
	function testWhereAnd()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+FROM\s+page\s+p'
			.'\s+INNER\s+JOIN\s+category\s+c'
			.'\s+ON\s+p.category_id=c.category_id'
			.'\s+AND\s+p.category_id=c.category_id'
			.'\s+WHERE\s+p\.name\s+LIKE\s+\?'
			.'\s+AND\s+c\.category_kbn=\?'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->and_('c.category_kbn=?' , '1')
				->toSQL()
		);
	}
	
	function testWhereAndParams()
	{
		$this->assertEqual(
			array('ページ名' , '1')
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->and_('c.category_kbn=?' , '1')
				->getParams()
		);
	}
	
	function testWhereOr()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+WHERE\s+p\.name\s+LIKE\s+\?'
			.'\s+AND\s+c\.category_kbn=\?'
			.'\s+OR\s+c\.category_kbn=\?'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->where('p.name LIKE ?' , 'ページ名')
				->and_('c.category_kbn=?' , '1')
				->or_('c.category_kbn=?' , '2')
				->toSQL()
		);
	}
	
	function testWhereOrParams()
	{
		$this->assertEqual(
			array('ページ名' , '1' , '2')
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->and_('c.category_kbn=?' , '1')
				->or_('c.category_kbn=?' , '2')
				->getParams()
		);
	}
	
	function testOrderBy()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+ORDER\s+BY\s+p\.update_date\s+DESC'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->and_('c.category_kbn=?' , '1')
				->or_('c.category_kbn=?' , '2')
				->orderBy('p.update_date DESC')
				->toSQL()
		);
	}
	
	function testLimit()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+ORDER\s+BY\s+p\.update_date\s+DESC'
			.'\s+LIMIT\s+10'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->and_('c.category_kbn=?' , '1')
				->or_('c.category_kbn=?' , '2')
				->orderBy('p.update_date DESC')
				->limit(10)
				->toSQL()
		);
	}
	
	function testOffset()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+ORDER\s+BY\s+p\.update_date\s+DESC'
			.'\s+LIMIT\s+10'
			.'\s+OFFSET\s+20'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->and_('c.category_kbn=?' , '1')
				->or_('c.category_kbn=?' , '2')
				->orderBy('p.update_date DESC')
				->limit(10)
				->offset(20)
				->toSQL()
		);
	}
	
	function testNestedWhere()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+WHERE\s+p\.name\s+LIKE\s+\?'
			.'\s+AND\s*\(\s*c\.category_kbn=\?'
			.'\s+OR\s+c\.category_kbn=\?\s*\)'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->and_(
					Wheremancy::_('c.category_kbn=?' , '1')
					->or_('c.category_kbn=?' , '2')
				)
				->toSQL()
		);
	}
	
	function testNestedWhereParams()
	{
		$this->assertEqual(
			array('ページ名' , '1' , '2')
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->join('CategoryDAO' , 'c')
				->on('p.category_id=c.category_id')
				->and_('p.category_id=c.category_id')
				->where('p.name LIKE ?' , 'ページ名')
				->and_(Wheremancy::_('c.category_kbn=?' , '1')
					->or_('c.category_kbn=?' , '2')
				)
				->getParams()
		);
	}
	
	function testParamsNull()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+FROM\s+page\s+p'
			.'\s+WHERE\s+c\.category_kbn=\?'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->where('p.name LIKE ?' , null)
				->and_('p.name=?' , array())
				->or_('p.name=?' , null)
				->and_('c.category_kbn=?' , 0)
				->toSQL()
		);
	}
	
	function testParamsNull2()
	{
		$this->assertPattern(
			'/^\s*SELECT'
			.'.+'
			.'\s+FROM\s+page\s+p'
			.'\s+WHERE\s+p\.name\s+LIKE\s+\?'
			.'\s+OR\s+c\.category_kbn=\?'
			.'/'
			, Daomancy::_($this->dbh)
				->from('PageDAO' , 'p')
				->where('p.name LIKE ?' , 'a%')
				->and_('p.name=?' , array())
				->or_('p.name=?' , array())
				->and_('p.name=?' , null)
				->or_('p.name=?' , null)
				->or_('c.category_kbn=?' , 0)
				->toSQL()
		);
	}
	
	
}

$test = new DaomancyTestCase();
$test->run(new HtmlReporter());

?>