<?php
set_include_path(get_include_path() . ';' . dirname(dirname(__FILE__)) );
error_reporting(E_ALL^E_NOTICE);

require_once 'simpletest/unit_tester.php';
require_once 'simpletest/reporter.php';

require_once 'daomancy/Daomancy.php';

class WheremancyTestCase extends UnitTestCase
{
	private $dbh;
	
	function setUp()
	{
	}
	
	function testWhere()
	{
		$this->assertPattern(
			'/^\s*WHERE\s+p\.name\s+LIKE\s+\?'
			.'/'
			, Wheremancy::_('p.name LIKE ?' , 'ページ名')
				->toSQL()
		);
	}
	
	function testWhereParams()
	{
		$this->assertEqual(
			array('ページ名')
			, Wheremancy::_('p.name LIKE ?' , 'ページ名')
				->getParams()
		);
	}
	
	function testArrayParams()
	{
		$this->assertEqual(
			array('p1' , 'p2' , 'p3' , 'p4')
			, Wheremancy::_('p.name IN(?,?,?,?)'
				, array('p1' , 'p2' , 'p3' , 'p4'))
				->getParams()
		);
	}
	
	function testAnd()
	{
		$this->assertPattern(
			'/^\s*WHERE\s+p\.name\s+LIKE\s+\?'
			.'\s+AND\s+c\.category_id=\?'
			.'/'
			, Wheremancy::_('p.name LIKE ?' , 'ページ名')
				->and_('c.category_id=?' , '1')
				->toSQL()
		);
	}
	
	function testOr()
	{
		$this->assertPattern(
			'/^\s*WHERE\s+p\.name\s+LIKE\s+\?'
			.'\s+OR\s+c\.category_id=\?'
			.'/'
			, Wheremancy::_('p.name LIKE ?' , 'ページ名')
				->or_('c.category_id=?' , '1')
				->toSQL()
		);
	}
	
	function testIn()
	{
		$this->assertPattern(
			'/^\s*WHERE\s+p\.name\s+IN\s*\(\?,\?,\?,\?\)'
			.'/'
			, Wheremancy::in('p.name' , array('p1' , 'p2' , 'p3' , 'p4'))
				->toSQL()
		);
	}
	
	function testIsNull()
	{
		$this->assertPattern(
			'/^\s*WHERE\s+p\.name\s+IS\s+NULL'
			.'/'
			, Wheremancy::isNull('p.name')
				->toSQL()
		);
	}
	
	function testIsNotNull()
	{
		$this->assertPattern(
			'/^\s*WHERE\s+p\.name\s+IS\s+NOT\s+NULL'
			.'/'
			, Wheremancy::isNotNull('p.name')
				->toSQL()
		);
	}
	
	function testNestedWhere()
	{
		$this->assertPattern(
			'/^\s*WHERE\s+p\.name=\?'
			.'\s+AND\s*\(p\.name=\?'
			.'\s+AND\s*\('
			.'\s*p\.name=\?\s+OR\s+p\.name=\?'
			.'\s*\)\s*\)'
			.'/'
			, Wheremancy::_('p.name=?' , 'n1')
				->and_(
					Wheremancy::_('p.name=?' , 'n2')
					->and_(
						Wheremancy::_('p.name=?' , 'n3')
						->or_('p.name=?' , 'n3or')
					)
				)
				->toSQL()
		);
	}
	
}

$test = new WheremancyTestCase();
$test->run(new HtmlReporter());

?>